/* EventHandlers.p */

USING System.Windows.Forms.* FROM ASSEMBLY.
USING Infragistics.Win.Misc.* FROM ASSEMBLY.

DEFINE VARIABLE rPointArray AS CLASS "System.Drawing.Point[]" NO-UNDO.
DEFINE VARIABLE rArray AS CLASS System.Array NO-UNDO.
DEFINE VARIABLE rPoint AS CLASS System.Drawing.Point NO-UNDO.
DEFINE VARIABLE rDrawBtn AS CLASS UltraButton NO-UNDO.
DEFINE VARIABLE rEraseBtn AS CLASS UltraButton NO-UNDO.
DEFINE VARIABLE rForm AS CLASS Progress.Windows.Form NO-UNDO.
DEFINE VARIABLE enColor AS CLASS System.Drawing.Color.
DEFINE VARIABLE sqrt2 AS DECIMAL NO-UNDO.
DEFINE VARIABLE idx AS INTEGER NO-UNDO.
FUNCTION AdjustOctagon RETURNS "System.Drawing.Point[]" FORWARD.

/* Create .NET Point array and a Point object to set its element values */
rPoint = NEW System.Drawing.Point( 0, 0 ).
rArray = System.Array:CreateInstance( rPoint:GetType( ), 8 ).
rPointArray = CAST (rArray, "System.Drawing.Point[]").

/* Prepare UI to draw octagons */
rDrawBtn = NEW UltraButton( ).
rDrawBtn:AutoSize = TRUE.
rDrawBtn:Text = "Draw".
rDrawBtn:DialogResult = DialogResult:None.
rDrawBtn:Top = 4.
rDrawBtn:Left = 4.

rEraseBtn = NEW UltraButton( ).
rEraseBtn:AutoSize = TRUE.
rEraseBtn:Text = "Erase".
rEraseBtn:DialogResult = DialogResult:None.
rEraseBtn:Top = 4.
rEraseBtn:Left = 10 + ( rDrawBtn:Width / 2 ).

rForm = NEW Progress.Windows.Form( ).
rForm:Height = 300.
rForm:Width = 300.
rForm:Text = "Octagons". 
rForm:Controls:Add( rDrawBtn ).
rForm:Controls:Add( rEraseBtn ).

sqrt2 = SQRT( 2.0 ). /* "Constant" needed to draw octagons */ 
enColor = System.Drawing.Color:Black. /* Initial pen color */

/* Event handlers that draw and redraw the octagon */
rEraseBtn:Click:Subscribe( "FormErase" ).
rDrawBtn:Click:Subscribe( "FormDraw" ).
rForm:Move:Subscribe( "FormMoveAndPaint" ).
rForm:Paint:Subscribe( "FormMoveAndPaint" ).

WAIT-FOR rForm:ShowDialog( ).

rForm:Dispose( ).

PROCEDURE Draw:

  /* Draw octagon from point array */
  DEFINE VARIABLE rGraphics AS CLASS System.Drawing.Graphics NO-UNDO.
  DEFINE VARIABLE rPen AS CLASS System.Drawing.Pen NO-UNDO.

  rGraphics = rForm:CreateGraphics( ).
  rPen = NEW System.Drawing.Pen( enColor ).
  rPen:Width = 2.

  /* Pass adjusted point array to .NET method to draw octagon */
  rGraphics:DrawPolygon( rPen, AdjustOctagon( ) ).

END PROCEDURE.

FUNCTION AdjustOctagon RETURNS "System.Drawing.Point[]":
  
  /* Adjust and return octagon point array according to form size */
  DEFINE VARIABLE iHorizA AS INTEGER NO-UNDO.
  DEFINE VARIABLE iHorizSide AS INTEGER NO-UNDO.
  DEFINE VARIABLE iVertA AS INTEGER NO-UNDO.
  DEFINE VARIABLE iVertSide AS INTEGER NO-UNDO.
  DEFINE VARIABLE iHoffset AS INTEGER NO-UNDO.
  DEFINE VARIABLE iVoffset AS INTEGER NO-UNDO.
  
  /* Calculate octagon size and position based on form client area */
  RUN CalcOctagonSide( INPUT rForm:ClientSize:Width, OUTPUT iHorizA, 
                       OUTPUT iHorizSide, OUTPUT iHoffset ). 
  RUN CalcOctagonSide( INPUT rForm:ClientSize:Height, OUTPUT iVertA, 
                       OUTPUT iVertSide, OUTPUT iVoffset ). 
    
  /* Generate octagon points */  
  DO idx = 0 TO 7:
    CASE idx:
      WHEN 0 THEN DO:
        rPoint:X = iHoffset + iHorizA.
        rPoint:Y = IVoffset.
      END.
      WHEN 1 THEN DO:
        rPoint:X =  iHoffset + iHorizA + iHorizSide.
        rPoint:Y = IVoffset.
      END.
      WHEN 2 THEN DO:
        rPoint:X = iHoffset + ( 2 * iHorizA ) + iHorizSide.
        rPoint:Y = iVoffset + iVertA.
      END.
      WHEN 3 THEN DO:
        rPoint:X = iHoffset + ( 2 * iHorizA ) + iHorizSide.
        rPoint:Y = iVoffset + iVertA + iVertSide.
      END.
      WHEN 4 THEN DO:
        rPoint:X =  iHoffset + iHorizA + iHorizSide.
        rPoint:Y = iVoffset + ( 2 * iVertA ) + iVertSide.
      END.
      WHEN 5 THEN DO:
        rPoint:X = iHoffset + iHorizA.
        rPoint:Y = iVoffset + ( 2 * iVertA ) + iVertSide.
      END.
      WHEN 6 THEN DO:
        rPoint:X = iHoffset.
        rPoint:Y = iVoffset + iVertA + iVertSide.
      END.
      WHEN 7 THEN DO:
        rPoint:X = iHoffset.
        rPoint:Y = iVoffset + iVertA.
      END.
    END CASE.    
    /* Replace point in point array */
    rPointArray:SetValue( rPoint, idx).
  END.  
  RETURN rPointArray.
  
END FUNCTION.

PROCEDURE CalcOctagonSide:

  DEFINE INPUT PARAMETER pDimension AS INTEGER NO-UNDO.
  DEFINE OUTPUT PARAMETER pA AS INTEGER NO-UNDO.
  DEFINE OUTPUT PARAMETER pSide AS INTEGER NO-UNDO.
  DEFINE OUTPUT PARAMETER pOffset AS INTEGER NO-UNDO.

  /* Calculate a side of a regular octagon with offsets */
  pA = 1 / ( ( 2 + sqrt2 ) / ( pDimension * 0.8 ) ).
  pSide = pA * sqrt2.
  pOffset = pDimension * 0.1.

END PROCEDURE.

PROCEDURE FormErase:

  DEFINE INPUT PARAMETER sender AS CLASS System.Object NO-UNDO.
  DEFINE INPUT PARAMETER e AS CLASS System.EventArgs NO-UNDO.

  enColor = rForm:BackColor.
  RUN Draw.

END PROCEDURE.

PROCEDURE FormDraw:

  DEFINE INPUT PARAMETER sender AS CLASS System.Object NO-UNDO.
  DEFINE INPUT PARAMETER e AS CLASS System.EventArgs NO-UNDO.

  enColor = System.Drawing.Color:Black.
  RUN Draw.

END PROCEDURE.

PROCEDURE FormMoveAndPaint:

  DEFINE INPUT PARAMETER sender AS CLASS System.Object NO-UNDO.
  DEFINE INPUT PARAMETER e AS CLASS System.EventArgs NO-UNDO.

  RUN Draw.

END PROCEDURE.