USING acme.myObjs.*.
USING acme.myObjs.Common.*.
USING acme.myObjs.Interfaces.*.

CLASS Main:

  DEFINE PRIVATE VARIABLE cOutFile     AS CHARACTER NO-UNDO.
  DEFINE PRIVATE VARIABLE rCommonObj   AS CLASS CommonObj NO-UNDO.
  DEFINE PRIVATE VARIABLE rCustObj     AS CLASS CustObj NO-UNDO.
  DEFINE PRIVATE VARIABLE rCustObj2    AS CLASS CustObj NO-UNDO.
  DEFINE PRIVATE VARIABLE rHelperClass AS CLASS HelperClass NO-UNDO.
  DEFINE PRIVATE VARIABLE rIBusObj     AS CLASS IBusObj NO-UNDO.

  /* first constructor instantiates a Customer object */
  CONSTRUCTOR PUBLIC Main ( ):                                     /* 10 */
    ASSIGN
      /* Create an instance of the HelperClass class  */
      rHelperClass = NEW HelperClass ( )
  
      /* Create an instance of the CustObj class      */
      rCustObj = NEW CustObj ( )
      cOutFile = "Customers.out".
                                                                   /* 16 */
    /* Subscribe OutputGenerated event handler for CustObj */
    rCustObj:OutputGenerated:Subscribe(OutputGenerated_CustObjHandler).
  END CONSTRUCTOR.

  /* second constructor takes a character parameter representing an input */
  /* file of email addresses to instantiate a New England Customer object */
  CONSTRUCTOR PUBLIC Main (INPUT EmailFile AS CHARACTER):          /* 10 */
    ASSIGN
      /* Create an instance of the HelperClass class */
      rHelperClass = NEW HelperClass( )
  
      /* Create an instance of the NECustObj class   */
      rCustObj = NEW NECustObj (EmailFile)
      cOutFile = "NECustomers.out".
                                                                   /* 16 */
    /* Subscribe OutputGenerated event handler for NECustObj */
    rCustObj:OutputGenerated:Subscribe(OutputGenerated_NECustObjHandler).
  END CONSTRUCTOR.

  /* Event handlers for for each Customer class instance */
  METHOD PRIVATE VOID OutputGenerated_CustObjHandler
      (pcOutputType AS CHARACTER):
    MESSAGE pcOutputType "for all customers." VIEW-AS ALERT-BOX.
  END METHOD.

  METHOD PRIVATE VOID OutputGenerated_NECustObjHandler
      (pcOutputType AS CHARACTER):
    MESSAGE pcOutputType "for New England customers." VIEW-AS ALERT-BOX.
  END METHOD.

  /* ObjectInfo processes information about the Customer object    */
  METHOD PUBLIC VOID ObjectInfo (piInfoCount AS INTEGER):
    /* Demonstrates passing object references as parameters      */

    rCommonObj = rCustObj.
                                                                   /* 10 */
    IF rCustObj:GetClass( ):TypeName = "acme.myObjs.NECustObj" THEN 
      MESSAGE "Initializing reports for New England customers at" 
        STRING(rCommonObj:updateTimestamp( ))                      /* 4 */
        VIEW-AS ALERT-BOX.
    ELSE
      MESSAGE "Initializing reports for all customers at" 
        STRING(rCommonObj:updateTimestamp( ))                      /* 4 */
        VIEW-AS ALERT-BOX.
      
    /* INPUT: it is valid to pass a subclass to a method         */
    /*        defined to take a super class                      */
    rHelperClass:InitializeDate (rCustObj).                        /* 4 */

    /* INPUT-OUTPUT: must be an exact match, a class to a method */
    /*               defined to take that same class type        */
    rHelperClass:ListNames (INPUT-OUTPUT rCustObj).
    rCustObj:CheckCredit ( ).
    rCustObj2 = rCustObj.

    /* OUTPUT: an interface is used to receive a class that      */
    /*         implements that interface                         */
    rHelperClass:ReportOutput (OUTPUT rIBusObj).
    IF piInfoCount <> ? AND piInfoCount > 1 THEN
      rIBusObj:printObj(piInfoCount).
    ELSE
      rIBusObj:printObj( ).
    rIBusObj:logObj (cOutFile).
    rIBusObj = rCustObj.
  END METHOD.
END CLASS.
