USING acme.myObjs.*.
USING acme.myObjs.Common.*.
USING acme.myObjs.Interfaces.*.

CLASS acme.myObjs.CustObj INHERITS CommonObj                       /* 1 */
                          IMPLEMENTS IBusObj:

  DEFINE PUBLIC VARIABLE iNumCusts AS INTEGER NO-UNDO.
  
  DEFINE PROTECTED TEMP-TABLE ttCust NO-UNDO
    FIELD RecNum AS INTEGER
    FIELD CustNum LIKE Customer.CustNum
    FIELD Name LIKE Customer.Name
    FIELD State AS CHARACTER.
  DEFINE PRIVATE VARIABLE rMsg AS CLASS MsgObj NO-UNDO.
  DEFINE PRIVATE VARIABLE rCreditObj AS CLASS CreditObj NO-UNDO.

  DEFINE PUBLIC OVERRIDE EVENT OutputGenerated                     /* 16 */
    SIGNATURE VOID (INPUT pcOutputType AS CHARACTER).

  METHOD PROTECTED OVERRIDE VOID PublishOutputGenerated            /* 16 */
    (INPUT pcOutputType AS CHARACTER):
    OutputGenerated:Publish(pcOutputType).
  END METHOD.    

  CONSTRUCTOR PUBLIC CustObj ( ):                                  /* 2 */
    rCreditObj = NEW CreditObj ( ).

    iNumCusts = 0.
    /* Fill temp table and get row count */
    FOR EACH Customer WHERE CreditLimit > 50000:
      CREATE ttCust.
      ASSIGN
        iNumCusts = iNumCusts + 1
        ttCust.RecNum = iNumCusts
        ttCust.CustNum = Customer.CustNum
        ttCust.Name = Customer.Name
        ttCust.State = Customer.State.
    END.
    rMsg = MessageHandler ("acme.myObjs.CustObj").                 /* 3 */
  END CONSTRUCTOR.
  
  METHOD PROTECTED OVERRIDE CLASS MsgObj MessageHandler            /* 3 */
    (INPUT piObjType AS CHARACTER):
    RETURN NEW MsgObj (piObjType).
  END METHOD.

  METHOD PUBLIC CHARACTER GetCustomerName (INPUT piRecNum AS INTEGER):
    FIND ttCust WHERE ttCust.RecNum = piRecNum NO-ERROR.           /* 15 */
    IF AVAILABLE ttCust THEN DO:
      RETURN ttCust.Name.
    END.
    ELSE DO:
      rMsg:Alert ("Customer number" + STRING(ttCust.RecNum) 
                  + "does not exist").
      RETURN ?.
    END.
  END METHOD.

  METHOD PUBLIC VOID CheckCredit ( ):
    IF VALID-OBJECT (rCreditObj) THEN DO:
      FOR EACH ttCust:
        rCreditObj:SetCurrentCustomer (ttCust.CustNum).
        rCreditObj:CheckCustCredit ( ).

        /* invokes the CustCreditLimit property GET accessor */
        rMsg:InfoMsg (ttCust.Name + " is in good standing." 
          + "  Credit Limit has been increased to " 
          + STRING(rCreditObj:CustCreditLimit)).                   /* 12 */

        CATCH e AS Progress.Lang.AppError:                         /* 13 */
          IF e:ReturnValue = "Over Limit" THEN DO:
            /* invokes the CustCreditLimit property GET accessor */
            rMsg:Alert (ttCust.Name + " is on Credit Hold." 
              + "  Balance exceeds Credit Limit of " 
              + STRING (rCreditObj:CustCreditLimit)).              /* 12 */
          END.
          ELSE
            rMsg:Alert ("Customer not found").
        END CATCH.
      END. /* FOR EACH */
    END.
    ELSE rMsg:Alert ("Unable to check credit").
  END METHOD.

  /* Must implement methods defined in the IBusObj interface      */
  /* dtTimestamp is a PROTECTED variable inherited from CommonObj */
  
  /* First version of printObj prints a single copy of a report */
  METHOD PUBLIC VOID printObj ( ):                                 /* 11 */
    OUTPUT TO PRINTER.
    DISPLAY dtTimestamp.                                           /* 5 */
    FOR EACH ttCust:
      DISPLAY ttCust.
    END.
    OUTPUT CLOSE.
    PublishOutputGenerated("One copy of report sent to printer").  /* 16 */
  END METHOD.

  /* Second version of printObj takes an integer parameter */
  /* representing the number of copies to print.           */
  METHOD PUBLIC VOID printObj (INPUT piCopies AS INTEGER):         /* 11 */
    DEFINE VARIABLE iCnt AS INTEGER.
    OUTPUT TO PRINTER.
    IF piCopies <> 0 THEN DO iCnt = 1 TO ABS(piCopies):
      DISPLAY dtTimestamp.                                         /* 5 */
      FOR EACH ttCust:
        DISPLAY ttCust.
      END.
    END.
    OUTPUT CLOSE.
    PublishOutputGenerated(STRING(piCopies)                        /* 16 */
                           + " copies of report sent to printer").
  END METHOD.
  
  /* Method to log customer information */
  METHOD PUBLIC VOID logObj (INPUT pcFilename AS CHARACTER):
    OUTPUT TO VALUE(pcFilename).
    DISPLAY dtTimestamp.                                           /* 5 */
    FOR EACH ttCust:
      DISPLAY ttCust.
    END.
    OUTPUT CLOSE.
    PublishOutputGenerated("One copy of report sent to "           /* 16 */
                           + pcFilename + " file").
  END METHOD.

  DESTRUCTOR PUBLIC CustObj ( ):                                   /* 6 */
    EMPTY TEMP-TABLE ttCust.
  END DESTRUCTOR.

END CLASS.
