USING acme.myObjs.*.

CLASS acme.myObjs.NECustObj INHERITS CustObj:                      /* 7 */
    
  DEFINE PRIVATE TEMP-TABLE ttEmail NO-UNDO
    FIELD RecNum AS INTEGER
    FIELD Name AS CHARACTER FORMAT "X(20)"
    FIELD Email AS CHARACTER FORMAT "X(20)".

  CONSTRUCTOR PUBLIC NECustObj (INPUT EmailFile AS CHARACTER):
    /* Because there are no parameters to the super class's */
    /* constructor, this constructor call is optional       */
    SUPER ( ).                                                     /* 8 */
    
    /* Code to initialize ttEmail:                        */
    /* The supplied file lists the email addresses in the */
    /* correct order for the customers being processed    */            
    INPUT FROM VALUE (EmailFile).
    FOR EACH ttCust 
      WHERE ttCust.State = "MA" OR                                 /* 14 */
            ttCust.State = "VT" OR
            ttCust.State = "NH" OR
            ttCust.State = "CT" OR
            ttCust.State = "RI" OR
            ttCust.State = "ME" :
      CREATE ttEmail.
      ASSIGN
        ttEmail.RecNum = ttCust.RecNum
        ttEmail.Name = ttCust.Name.
      IMPORT ttEmail.Email.
    END.
  END CONSTRUCTOR.

  /* Override method to always get customer name and email */
  METHOD PUBLIC OVERRIDE CHARACTER GetCustomerName 
    (INPUT piCustNum AS INTEGER):                                  /* 9 */
    DEFINE VARIABLE EmailName AS CHARACTER NO-UNDO.

    EmailName = SUPER:GetCustomerName (piCustNum).
    FIND FIRST ttEmail WHERE ttEmail.Name = EmailName NO-ERROR.    /* 15 */
    IF AVAILABLE (ttEmail) THEN
      RETURN EmailName + ";" + ttEmail.Email.
    ELSE
      RETURN EmailName.
  END METHOD.

  /* First override version of printObj for a single copy */
  METHOD PUBLIC OVERRIDE VOID printObj ( ):                        /* 11 */
    OUTPUT TO PRINTER.
    DISPLAY dtTimestamp.                                           /* 5 */
    FOR EACH ttEmail:
      DISPLAY ttEmail.
    END.
    OUTPUT CLOSE.
    PublishOutputGenerated("One copy of report sent to printer").  /* 16 */
  END METHOD.

  /* Second override version of printObj for multiple copies */
  METHOD PUBLIC OVERRIDE VOID printObj 
    (INPUT piCopies AS INTEGER):                                   /* 11 */
    DEFINE VARIABLE iCnt AS INTEGER.
    OUTPUT TO PRINTER.
    IF piCopies <> 0 THEN DO iCnt = 1 TO ABS(piCopies):
      DISPLAY dtTimestamp.                                         /* 5 */
      FOR EACH ttEmail:
        DISPLAY ttEmail.
      END.
    END.
    OUTPUT CLOSE.
    PublishOutputGenerated(STRING(piCopies)                        /* 16 */
                           + " copies of report sent to printer").
  END METHOD.

  /* Override method to log customer email information */
  METHOD PUBLIC OVERRIDE VOID logObj (INPUT pcFilename AS CHARACTER):
    OUTPUT TO VALUE (pcFilename).
    DISPLAY dtTimestamp.                                           /* 5 */
    FOR EACH ttEmail:
      DISPLAY ttEmail.
    END.
    OUTPUT CLOSE.
    PublishOutputGenerated("One copy of report sent to "           /* 16 */
                           + pcFilename + " file").
  END METHOD.

END CLASS.
