/* Main Block */                                                   /* 2 */

DEFINE NEW SHARED VARIABLE dtTimestamp AS DATETIME NO-UNDO.        /* 5 */
DEFINE     SHARED VARIABLE iNumCusts AS INTEGER  NO-UNDO.

DEFINE VARIABLE hCommon     AS HANDLE NO-UNDO.
DEFINE VARIABLE hCreditProc AS HANDLE NO-UNDO.
DEFINE VARIABLE hMsg        AS HANDLE NO-UNDO.

DEFINE TEMP-TABLE ttCustomer NO-UNDO                               /* 14 */
  FIELD RecNum  AS INTEGER
  FIELD CustNum LIKE Customer.CustNum
  FIELD Name    LIKE Customer.Name 
  FIELD State   AS CHARACTER.

RUN CommonProc.p PERSISTENT SET hCommon.                           /* 1 */
THIS-PROCEDURE:ADD-SUPER-PROCEDURE(hCommon).

RUN CreditProc.p PERSISTENT SET hCreditProc.

FUNCTION GetCreditLimit RETURNS INTEGER ( ) IN hCreditProc.
FUNCTION MessageHandler RETURNS HANDLE 
  (INPUT ProcType AS CHARACTER) IN SUPER.

/* Fill temp table and get row count */
FOR EACH Customer WHERE CreditLimit > 50000:
  CREATE ttCustomer.
  ASSIGN
    iNumCusts          = iNumCusts + 1
    ttCustomer.RecNum  = iNumCusts
    ttCustomer.CustNum = Customer.CustNum
    ttCustomer.Name    = Customer.Name
    ttCustomer.State   = Customer.State.
END.
hMsg = MessageHandler(INPUT "CustProc").                           /* 3 */

PROCEDURE GetTT:                                                   /* 14 */
  DEFINE OUTPUT PARAMETER TABLE FOR ttCustomer BIND.
END PROCEDURE.

PROCEDURE GetCustomerName:
  DEFINE INPUT  PARAMETER piRecNum AS INTEGER   NO-UNDO.
  DEFINE OUTPUT PARAMETER poName   AS CHARACTER NO-UNDO.

  FIND ttCustomer WHERE ttCustomer.RecNum = piRecNum NO-ERROR.     /* 15 */
  IF AVAILABLE ttCustomer THEN
    poName = ttCustomer.Name.
  ELSE DO:
    RUN Alert IN hMsg("Customer number" + STRING(ttCustomer.RecNum) 
                      + "does not exist").
    poName = ?.
  END.
END PROCEDURE.

PROCEDURE CheckCredit:
  IF VALID-HANDLE(hCreditProc) THEN DO:
    FOR EACH ttCustomer:
      RUN SetCurrentCustomer IN hCreditProc (ttCustomer.CustNum).
      RUN CheckCustCredit IN hCreditProc.

      RUN InfoMsg IN hMsg (ttCustomer.Name + " is in good standing." +
        "  Credit Limit has been increased to " +
        STRING (GetCreditLimit( )) ).                             /* 12 */

      CATCH e AS Progress.Lang.AppError:                          /* 13 */
        IF e:ReturnValue = "Over Limit" THEN
          RUN Alert IN hMsg (ttCustomer.Name + " is on Credit Hold." +
            "  Balance exceeds Credit Limit of " +
            STRING(GetCreditLimit( ))).    /* 12 */
        ELSE
          RUN Alert IN hMsg ("Customer not found").
      END CATCH.
    END. /* FOR EACH */
  END.
  ELSE 
    RUN Alert IN hMsg ("Unable to check credit").
END PROCEDURE.

/* dtTimestamp is a SHARED variable defined in CommonProc.p as well */
PROCEDURE printProc:                                               /* 11 */
  DEFINE INPUT PARAMETER piCopies AS INTEGER NO-UNDO.
  DEFINE VARIABLE iCnt AS INTEGER.
  OUTPUT TO PRINTER.
  IF piCopies <> 0 THEN DO iCnt = 1 TO ABS(piCopies):
    DISPLAY dtTimestamp.                                           /* 5 */
    FOR EACH ttCustomer:
      DISPLAY ttCustomer.
    END.
  END.
  OUTPUT CLOSE.
  IF ABS(piCopies) > 1 THEN                                        /* 16 */
    PUBLISH "OutputGenerated" (STRING(piCopies) 
                               + " copies of report sent to printer").
  ELSE
    PUBLISH "OutputGenerated" ("One copy of report sent to printer").
END PROCEDURE.

PROCEDURE logProc:
  DEFINE INPUT PARAMETER pcFilename AS CHARACTER NO-UNDO.

  OUTPUT TO VALUE(pcFilename).
  DISPLAY dtTimestamp.                                             /* 5 */
  FOR EACH ttCustomer:
    DISPLAY ttCustomer.
  END.
  OUTPUT CLOSE.
  PUBLISH "OutputGenerated" ("One copy of report sent to "         /* 16 */
                             + pcFilename + " file").
END PROCEDURE.

PROCEDURE CleanUp:
  EMPTY TEMP-TABLE ttCustomer.
  DELETE OBJECT hMsg.
  DELETE OBJECT hCreditProc.
  RUN CleanUp IN hCommon.
  DELETE OBJECT hCommon.
END PROCEDURE.
