DEFINE INPUT PARAMETER EmailFile AS CHARACTER NO-UNDO.

DEFINE NEW SHARED VARIABLE dtTimestamp AS DATETIME NO-UNDO.

DEFINE VARIABLE hCustProc   AS HANDLE NO-UNDO.
DEFINE VARIABLE hCommonProc AS HANDLE NO-UNDO.

DEFINE TEMP-TABLE ttCustomer NO-UNDO REFERENCE-ONLY                /* 14 */
  FIELD RecNum  AS INTEGER
  FIELD CustNum LIKE Customer.CustNum
  FIELD Name    LIKE Customer.Name
  FIELD State   AS CHARACTER. 

DEFINE TEMP-TABLE ttEmail NO-UNDO
  FIELD RecNum AS INTEGER
  FIELD Name   AS CHARACTER FORMAT "X(20)"
  FIELD Email  AS CHARACTER FORMAT "X(20)".

/* Super procedures are searched in LIFO order */
RUN CommonProc.p PERSISTENT SET hCommonProc.                       /* 8 */
THIS-PROCEDURE:ADD-SUPER-PROCEDURE(hCommonProc).                   /* 7 */
RUN CustProc.p PERSISTENT SET hCustProc.                           /* 8 */
THIS-PROCEDURE:ADD-SUPER-PROCEDURE(hCustProc).                     /* 7 */

RUN GetTT (OUTPUT TABLE ttCustomer BIND).

INPUT FROM VALUE(EmailFile).
FOR EACH ttCustomer 
  WHERE ttCustomer.State = "MA" OR                                 /* 14 */
    ttCustomer.State = "VT" OR
    ttCustomer.State = "NH" OR
    ttCustomer.State = "CT" OR
    ttCustomer.State = "RI" OR
    ttCustomer.State = "ME":
  CREATE ttEmail.
  ASSIGN
    ttEmail.RecNum = ttCustomer.RecNum
    ttEmail.Name   = ttCustomer.Name.
  IMPORT ttEmail.Email.
END.
INPUT CLOSE.

PROCEDURE GetCustomerName:                                         /* 9 */
  DEFINE INPUT  PARAMETER piRecNum AS INTEGER   NO-UNDO.
  DEFINE OUTPUT PARAMETER poName   AS CHARACTER NO-UNDO.

  RUN SUPER(INPUT piRecNum, OUTPUT poName).
  FIND FIRST ttEmail WHERE ttEmail.Name = poName NO-ERROR.         /* 15 */
  IF AVAILABLE(ttEmail) THEN
    RETURN poName + ";" + ttEmail.Email.
END PROCEDURE.

/* dtTimestamp is a SHARED variable defined in CommonProc.p as well */
PROCEDURE printProc:                                               /* 11 */
  DEFINE INPUT PARAMETER piCopies AS INTEGER NO-UNDO.
  DEFINE VARIABLE iCnt AS INTEGER.
  OUTPUT TO PRINTER.
  IF piCopies <> 0 THEN DO iCnt = 1 TO ABS(piCopies):
    DISPLAY dtTimestamp.                                           /* 5 */
    FOR EACH ttEmail:
      DISPLAY ttEmail.
    END.
  END.
  OUTPUT CLOSE.
  IF ABS(piCopies) > 1 THEN                                        /* 16 */
    PUBLISH "OutputGenerated" (STRING(piCopies) 
                               + " copies of report sent to printer").
  ELSE
    PUBLISH "OutputGenerated" ("One copy of report sent to printer").
END PROCEDURE.

PROCEDURE logProc:
  DEFINE INPUT PARAMETER pcFilename AS CHARACTER NO-UNDO.

  OUTPUT TO VALUE(pcFilename).
  DISPLAY dtTimestamp.                                             /* 5 */
  FOR EACH ttEmail:
    DISPLAY ttEmail.
  END.
  OUTPUT CLOSE.
  PUBLISH "OutputGenerated" ("One copy of report sent to "         /* 16 */
                             + pcFilename + " file").
END PROCEDURE.