/* h-InvSummary.p - Uses a temp-table to build a summary report of 
   Invoices by Customer. */

DEFINE TEMP-TABLE ttInvoice NO-UNDO
  FIELD iCustNum   LIKE Invoice.CustNum LABEL "Cust#" FORMAT "ZZ9"
  FIELD cCustName  LIKE Customer.NAME FORMAT "X(20)"
  FIELD iNumInvs   AS INTEGER LABEL "# Inv's" FORMAT "Z9"
  FIELD dInvTotal  AS DECIMAL LABEL "Inv Total" FORMAT ">>,>>9.99" 
  FIELD dMaxAmount AS DECIMAL LABEL "Max Amount" FORMAT ">>,>>9.99"
  FIELD iInvNum    LIKE Invoice.InvoiceNum LABEL "Inv#" FORMAT "ZZ9"
  INDEX idxCustNum IS PRIMARY iCustNum
  INDEX idxInvTotal dInvTotal.

/* Retrieve each invoice along with its Customer record, to get the Name. */
FOR EACH Invoice NO-LOCK, Customer OF Invoice NO-LOCK:
  FIND FIRST ttInvoice WHERE ttInvoice.iCustNum = Invoice.CustNum NO-ERROR.
  /* If there isn't already a temp-table record for the Customer, create it 
     and save the Customer # and Name. */
  IF NOT AVAILABLE ttInvoice THEN DO:
    CREATE ttInvoice.
    ASSIGN ttInvoice.iCustNum  = Invoice.CustNum
           ttInvoice.cCustName = Customer.Name.
  END.

  /* Save off the Invoice amount if it's a new high for this Customer. */
  IF Invoice.Amount > dMaxAmount THEN
    ASSIGN dMaxAmount = Invoice.Amount
           iInvNum    = Invoice.InvoiceNum.
  /* Increment the Invoice total and Invoice count for the Customer. */
  ASSIGN ttInvoice.dInvTotal = ttInvoice.dInvTotal + Invoice.Amount
         ttInvoice.iNumInvs  = ttInvoice.iNumInvs + 1.
END. /* FOR EACH Invoice, Customer OF Invoice */

/* Display the results in descending order by invoice total. */
FOR EACH ttInvoice BY dInvTotal DESCENDING:
  DISPLAY iCustNum cCustName iNumInvs dInvTotal iInvNum dMaxAmount.
END.
