&ANALYZE-SUSPEND _VERSION-NUMBER UIB_v9r12
&ANALYZE-RESUME
&ANALYZE-SUSPEND _UIB-CODE-BLOCK _CUSTOM _DEFINITIONS Procedure 
/*------------------------------------------------------------------------
    File        : h-gettemptable.p
    Purpose     : Demonstrates how to pass temp-table handles from server
                  to client.

    Author(s)   : Bruce Gruenbaum
    
    Notes       : This file contains sample code which may assist you in 
                  creating applications.  You may use the code as you see 
                  fit. If you modify the code or include it in another 
                  software program, you will refrain from identifying 
                  Progress Software as the supplier of the code, or using 
                  any Progress Software trademarks in connection with 
                  your use of the code. 
                  
                  THE CODE IS NOT SUPPORTED BY PROGRESS SOFTWARE AND IS 
                  PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND, 
                  INCLUDING, WITHOUT LIMITATION, ANY WARRANTY OF 
                  MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR 
                  NONINFRINGEMENT.
  ----------------------------------------------------------------------*/
/*          This .W file was created with the Progress AppBuilder.      */
/*----------------------------------------------------------------------*/

/* ***************************  Definitions  ************************** */
DEFINE INPUT  PARAMETER icTables AS CHARACTER  NO-UNDO.
DEFINE INPUT  PARAMETER icForEach AS CHARACTER  NO-UNDO.
DEFINE INPUT  PARAMETER icFields AS CHARACTER  NO-UNDO.
DEFINE INPUT  PARAMETER icTableName AS CHARACTER  NO-UNDO.
DEFINE INPUT  PARAMETER iiRows AS INTEGER    NO-UNDO.



/* This is the parameter that is so cool */
DEFINE OUTPUT PARAMETER TABLE-HANDLE  ohTable /* hTT */.

/* cBuffers stores a list of buffer handles that maps to icTables */
DEFINE VARIABLE cBuffers AS CHARACTER NO-UNDO.

/* cTableFields has the list of Field handles that maps to icFields */
DEFINE VARIABLE cTableFields AS CHARACTER NO-UNDO.

/* hTT is the handle to the dynamic temp table */
DEFINE VARIABLE hTT AS HANDLE     NO-UNDO. 

/* hQuery is the handle to the query */
DEFINE VARIABLE hQuery AS HANDLE NO-UNDO.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME


&ANALYZE-SUSPEND _UIB-PREPROCESSOR-BLOCK 

/* ********************  Preprocessor Definitions  ******************** */

&Scoped-define PROCEDURE-TYPE Procedure
&Scoped-define DB-AWARE no



/* _UIB-PREPROCESSOR-BLOCK-END */
&ANALYZE-RESUME



/* *********************** Procedure Settings ************************ */

&ANALYZE-SUSPEND _PROCEDURE-SETTINGS
/* Settings for THIS-PROCEDURE
   Type: Procedure
   Allow: 
   Frames: 0
   Add Fields to: Neither
   Other Settings: CODE-ONLY COMPILE
 */
&ANALYZE-RESUME _END-PROCEDURE-SETTINGS

/* *************************  Create Window  ************************** */

&ANALYZE-SUSPEND _CREATE-WINDOW
/* DESIGN Window definition (used by the UIB) 
  CREATE WINDOW Procedure ASSIGN
         HEIGHT             = 15
         WIDTH              = 60.
/* END WINDOW DEFINITION */
                                                                        */
&ANALYZE-RESUME

 


&ANALYZE-SUSPEND _UIB-CODE-BLOCK _CUSTOM _MAIN-BLOCK Procedure 


/* ***************************  Main Block  *************************** */

/* Create a query - we're doing this here because the buffers need to be 
   added in createBuffers  */
CREATE QUERY hQuery.

/* Create the buffers and add them to the query */
RUN createBuffers.

/* Get a list of field handles for the fields */
RUN getFieldHandles.

/* Create and prepare the temp-table */
RUN createTempTable.

/* Open the query and copy the data */
RUN populateTempTable.

/* Delete all the objects */
RUN cleanupQuery.

/* Set the output table handle to the temp-table */
ohTable = hTT. 

/*
DELETE OBJECT hTT.
  */

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME


/* **********************  Internal Procedures  *********************** */

&IF DEFINED(EXCLUDE-cleanupQuery) = 0 &THEN

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _PROCEDURE cleanupQuery Procedure 
PROCEDURE cleanupQuery :
/*------------------------------------------------------------------------------
  Purpose:     Deletes the query and all its buffers 
  Parameters:  <none>
  Notes:       
------------------------------------------------------------------------------*/
  DEFINE VARIABLE iCount AS INTEGER    NO-UNDO.
  DEFINE VARIABLE hBuffer AS HANDLE     NO-UNDO.
  
  /* Delete the query object */
  DELETE OBJECT hQuery.

  /* Iterate through the list of buffers */
  DO iCount = 1 TO NUM-ENTRIES(cBuffers):
    /* Convert the string to a handle */
    hBuffer = WIDGET-HANDLE(ENTRY(iCount,cBuffers)).

    /* Delete the buffer */
    DELETE OBJECT hBuffer.
  END.

END PROCEDURE.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ENDIF

&IF DEFINED(EXCLUDE-createBuffers) = 0 &THEN

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _PROCEDURE createBuffers Procedure 
PROCEDURE createBuffers :
/*------------------------------------------------------------------------------
  Purpose:     Creates buffers and adds them to the query
  Parameters:  <none>
  Notes:       
------------------------------------------------------------------------------*/

  DEFINE VARIABLE iCount AS INTEGER    NO-UNDO.
  DEFINE VARIABLE hBuffer AS HANDLE     NO-UNDO.
  
  /* Iterate through the list of table names */
  DO iCount = 1 TO NUM-ENTRIES(icTables):
    /* Create a buffer for each table name */
    CREATE BUFFER hBuffer FOR TABLE ENTRY(iCount,icTables).
    /* String the handles of the buffers into cBuffers */
    cBuffers = cBuffers + (IF cBuffers <> "" THEN "," ELSE "")
             + STRING(hBuffer).
    /* Add the buffer to the query */
    hQuery:ADD-BUFFER(hBuffer).
  END.
END PROCEDURE.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ENDIF

&IF DEFINED(EXCLUDE-createTempTable) = 0 &THEN

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _PROCEDURE createTempTable Procedure 
PROCEDURE createTempTable :
/*------------------------------------------------------------------------------
  Purpose:     Creates the temp-table with the fields that the user requested 
  Parameters:  <none>
  Notes:       
------------------------------------------------------------------------------*/
  DEFINE VARIABLE iCount AS INTEGER    NO-UNDO.
  DEFINE VARIABLE hField AS HANDLE     NO-UNDO.
  DEFINE VARIABLE cField AS CHARACTER  NO-UNDO.

  /* Create the temp table */
  CREATE TEMP-TABLE hTT.

  /* Add a count field - we need this for the sequence so that we get the records
     back in the order that they were created */
  hTT:ADD-NEW-FIELD("iCount","INTEGER").

  /* Iterate through the list of field handles in the cTableFields */
  DO iCount = 1 TO NUM-ENTRIES(cTableFields):
    /* Convert the current entry to a handle */
    hField = WIDGET-HANDLE(ENTRY(iCount,cTableFields)).
    /* Add a field to the temp table like this field with the same name as this 
       field */
    hTT:ADD-LIKE-FIELD(hField:NAME,hField).
  END.

  /* Add a primary unique index to the temp-table */
  hTT:ADD-NEW-INDEX("pudx",True,True).

  /* Add the counter field to the index */
  hTT:ADD-INDEX-FIELD("pudx","iCount").

  /* Prepare the temp-table with the name the user chose */
  hTT:TEMP-TABLE-PREPARE(icTableName).
  
END PROCEDURE.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ENDIF

&IF DEFINED(EXCLUDE-getFieldHandles) = 0 &THEN

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _PROCEDURE getFieldHandles Procedure 
PROCEDURE getFieldHandles :
/*------------------------------------------------------------------------------
  Purpose:     Gets the list of field handles from the buffer 
  Parameters:  <none>
  Notes:       
------------------------------------------------------------------------------*/
  DEFINE VARIABLE iCount AS INTEGER NO-UNDO.
  DEFINE VARIABLE hField AS HANDLE NO-UNDO.
  DEFINE VARIABLE iBuffCount AS INTEGER NO-UNDO.
  DEFINE VARIABLE cField AS CHARACTER NO-UNDO.
  DEFINE VARIABLE cBuffer AS CHARACTER NO-UNDO.
  DEFINE VARIABLE hBuffer AS HANDLE NO-UNDO.

  /* Iterate through the list of field names */
  DO iCount = 1 TO NUM-ENTRIES(icFields):

    /* We expect that all field names will be in the <table>.<field> format */

    /* Get the entry into the cBuffer field */
    cBuffer = ENTRY(iCount,icFields).
    /* Take the actual field name and store it in the cField variable */
    cField = ENTRY(2,cBuffer,".").
    /* Store only the table name in the cBuffer field */
    cBuffer = ENTRY(1,cBuffer,".").

    /* Get the handle of the buffer that matches this table name */
    hBuffer = WIDGET-HANDLE(ENTRY(LOOKUP(cBuffer,icTables),cBuffers)).

    /* Get a handle to the field that matches the field name */
    hField = hBuffer:BUFFER-FIELD(cField).

    /* String this handle into the list of fields */
    cTableFields = cTableFields + (IF cTableFields <> "" THEN "," ELSE "")
             + STRING(hField).
    
  END.

END PROCEDURE.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ENDIF

&IF DEFINED(EXCLUDE-populateTempTable) = 0 &THEN

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _PROCEDURE populateTempTable Procedure 
PROCEDURE populateTempTable :
/*------------------------------------------------------------------------------
  Purpose:     Opens the query and copies the appropriate data to the temp-table
  Parameters:  <none>
  Notes:       
------------------------------------------------------------------------------*/
  DEFINE VARIABLE iCount AS INTEGER NO-UNDO.
  DEFINE VARIABLE iCount2 AS INTEGER NO-UNDO.
  DEFINE VARIABLE hTTBuff AS HANDLE NO-UNDO.
  DEFINE VARIABLE hField AS HANDLE NO-UNDO.
  DEFINE VARIABLE hFromField AS HANDLE NO-UNDO.
  
  /* Store the handle to the buffer for the temp-table */
  hTTBuff = hTT:DEFAULT-BUFFER-HANDLE.

  /* Prepare a query using the string the user provided */
  hQuery:QUERY-PREPARE(icForEach).
  /* Open the query */
  hQuery:QUERY-OPEN().
  /* Get the first result in the result set */
  hQuery:GET-FIRST().

  /* Iterate for as many rows as the user has chosen, or until the query is 
     off-end */
  REPEAT iCount = 1 TO iiRows WHILE NOT hQuery:QUERY-OFF-END:
   
    /* Create a temp-table record */
    hTTBuff:BUFFER-CREATE().
    /* Iterate through the fields in the temp table */
    DO iCount2 = 1 TO hTTBuff:NUM-FIELDS:
      /* Get the handle to the current field */
      hField = hTTBuff:BUFFER-FIELD(iCount2).
      /* If this is the first field, we just increment the counter */
      IF iCount2 = 1 THEN
        hField:BUFFER-VALUE = iCount.
      ELSE
      DO:
        /* Otherwise get the handle of the corresponding field in the 
           cTableFields list */
        hFromField = WIDGET-HANDLE(ENTRY(iCount2 - 1,cTableFields)).
        /* Set the value of the temp-table field to the value of the 
           database table field */
        hField:BUFFER-VALUE = hFromField:BUFFER-VALUE.
      END.
    END.
    
    /* Release the temp-table record */
    hTTBuff:BUFFER-RELEASE().
    /* Get teh next result in the query */
    hQuery:GET-NEXT().
  END.
  /* Close the query */
  hQuery:QUERY-CLOSE().
  
END PROCEDURE.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ENDIF

