/* h-TestDynTT.p -- test dynamic temp-table methods. */
DEFINE VARIABLE cName  LIKE Customer.NAME    NO-UNDO.
DEFINE VARIABLE cRep   LIKE SalesRep.RepName NO-UNDO.
DEFINE VARIABLE hQuery AS HANDLE             NO-UNDO.
DEFINE VARIABLE hTT    AS HANDLE             NO-UNDO.
DEFINE VARIABLE hTTBuf AS HANDLE             NO-UNDO.
DEFINE VARIABLE iCNum  LIKE Customer.CustNum NO-UNDO.
DEFINE VARIABLE iSeq   AS INTEGER            NO-UNDO INITIAL 1000.

CREATE TEMP-TABLE hTT.
hTT:CREATE-LIKE("Customer","Name").
hTT:ADD-FIELDS-FROM("SalesRep","MonthQuota").
hTT:ADD-LIKE-FIELD("Area", "SalesRep.Region").
hTT:ADD-NEW-FIELD("Sequence", "INTEGER",0,"9999",1000).
hTT:ADD-LIKE-INDEX("CustNum","CustNum","Customer").
hTT:ADD-NEW-INDEX("SeqIndex", TRUE).  /* Make it the primary index */
hTT:ADD-INDEX-FIELD("SeqIndex", "Sequence").
hTT:TEMP-TABLE-PREPARE("CustSequence").
hTTBuf = hTT:DEFAULT-BUFFER-HANDLE.

/* Populate the temp-table with values from the database. */
FOR EACH Customer WHERE Customer.State = "NH" NO-LOCK,
  SalesRep OF Customer NO-LOCK:
  hTTBuf:BUFFER-CREATE().
  hTTBuf:BUFFER-COPY(BUFFER Customer:HANDLE).
  hTTBuf:BUFFER-COPY(BUFFER SalesRep:HANDLE).
  hTTBuf:BUFFER-FIELD("Sequence"):BUFFER-VALUE = iSeq.
  iSeq = iSeq + 1.
END.

/* Now create a query for the temp-table buffer and display values. */
CREATE QUERY hQuery.
hQuery:SET-BUFFERS(hTTBuf).
hQuery:QUERY-PREPARE("FOR EACH CustSequence").
hQuery:QUERY-OPEN().
hQuery:GET-FIRST().
REPEAT WHILE NOT hQuery:QUERY-OFF-END:
  ASSIGN 
    iSeq  = hTTBuf:BUFFER-FIELD("Sequence"):BUFFER-VALUE
    cName = hTTBuf:BUFFER-FIELD("Name"):BUFFER-VALUE
    iCNum = hTTBuf:BUFFER-FIELD("CustNum"):BUFFER-VALUE
    cRep  = hTTBuf:BUFFER-FIELD("RepName"):BUFFER-VALUE.
  DISPLAY 
    iSeq FORMAT "9999" LABEL "Sequence"
    iCNum 
    cName FORMAT "X(20)"
    cRep FORMAT "X(20)" WITH FRAME CustSeqFrame 12 DOWN.
  hQuery:GET-NEXT().
END.



