/* i-sax2h.p - So we can create new records */

/* Variable in which to accumulate all the text data for one element
   coming in through potentially multiple calls (per element) to the
   Characters procedure */
DEFINE VARIABLE currentFieldValue AS CHARACTER NO-UNDO.
DEFINE VARIABLE hBuf              AS HANDLE    NO-UNDO.
DEFINE VARIABLE hDBFld            AS HANDLE    NO-UNDO.
DEFINE VARIABLE iProcessingState  AS INTEGER   NO-UNDO.

DEFINE TEMP-TABLE ttCustomer LIKE Customer.

hBuf = BUFFER ttCustomer:HANDLE.

/* Simple-minded state machine. The code makes minimal use of it, but it could
   easily be used to validate the structure of the document in this example */
&SCOPED-DEFINE READY-TO-START 1
&SCOPED-DEFINE GETTING-RECORDS 2
&SCOPED-DEFINE GETTING-FIELDS 3
&SCOPED-DEFINE DONE 4

PROCEDURE StartDocument:
  iProcessingState = {&READY-TO-START}.
END.

PROCEDURE StartElement:
  DEFINE INPUT PARAMETER namespaceURI AS CHARACTER NO-UNDO.
  DEFINE INPUT PARAMETER localName    AS CHARACTER NO-UNDO.
  DEFINE INPUT PARAMETER qName        AS CHARACTER NO-UNDO.
  DEFINE INPUT PARAMETER attributes   AS HANDLE    NO-UNDO.

  IF qName = "Customers" THEN
    iProcessingState = {&GETTING-RECORDS}.
  ELSE IF qName = "Customer" THEN DO:
    /* Starting a new customer, so create the record */
    CREATE ttCustomer.
    
    /* Get the fields that are in the XML doc as attributes*/
    ASSIGN
      ttCustomer.CustNum = INTEGER(attributes:GET-VALUE-BY-QNAME("CustNum"))
      ttCustomer.Name    = attributes:GET-VALUE-BY-QNAME("Name")
      iProcessingState   = {&GETTING-FIELDS}.
  END.
  ELSE IF iProcessingState = {&GETTING-FIELDS} THEN DO:
    /* Get a handle to the field whose name corresponds to the element name */
    hDBFld = hBuf:BUFFER-FIELD(qName).
    
    /* Re-init the variable in which we accumulate the field value */
    currentFieldValue = "".
  END.
END.
PROCEDURE Characters:
  DEFINE INPUT PARAMETER charData AS MEMPTR  NO-UNDO.
  DEFINE INPUT PARAMETER numChars AS INTEGER NO-UNDO.
  
  /* Get the text value of the field (hDBFld was set to the correct field
     in StartElement */
  currentFieldValue = currentFieldValue + GET-STRING(charData, 1, GET-SIZE(charData)).
END.

PROCEDURE EndElement:
  DEFINE INPUT PARAMETER namespaceURI AS CHARACTER NO-UNDO.
  DEFINE INPUT PARAMETER localName    AS CHARACTER NO-UNDO.
  DEFINE INPUT PARAMETER qName        AS CHARACTER NO-UNDO.

  IF localName = "Customers" THEN
    iProcessingState = {&DONE}.
  ELSE IF localName = "Customer"THEN
    iProcessingState = {&GETTING-RECORDS}.
  ELSE IF iProcessingState = {&GETTING-FIELDS} THEN
    hDBFld:BUFFER-VALUE = currentFieldValue.
END.

PROCEDURE EndDocument:
  /* Show that data made it by displaying temp-table */
  FOR EACH ttCustomer:
    DISPLAY ttCustomer.NAME.
  END.
  RUN Cleanup.
END.

PROCEDURE FatalError:
  DEFINE INPUT PARAMETER errMessage AS CHARACTER NO-UNDO.

  /* Not necessary to do anything with PRIVATE-DATA, this is
     just an example of what you could do */
  SELF:PRIVATE-DATA = "FATAL".
  RUN Cleanup.
  
  /* RETURN ERROR in an error handler implicitly calls SELF:STOP-PARSING(),
     sets SELF:PARSE-STATUS to SAX-PARSER-ERROR, and raises the Progress
     ERROR condition */
  RETURN ERROR errMessage
    + "(Line " + STRING(SELF:LOCATOR-LINE-NUMBER)
    + ", Col " + STRING(SELF:LOCATOR-COLUMN-NUMBER) + ")".
END.

/* This is not a SAX callback, it is just a local utility */
PROCEDURE Cleanup:
  /* In case we've parsed previous documents */
  hBuf:EMPTY-TEMP-TABLE( ).
END.
