 /*------------------------------------------------------------------------
    File        : Service
    Purpose     : 
    Syntax      : 
    Description : 
    Author(s)   : 
    Created     : Tue Jul 03 13:41:11 EDT 2007
    Notes       : 
  ----------------------------------------------------------------------*/
using openedge.tutorial.services.*.

CLASS openedge.tutorial.services.Service   :
    define private static variable fService as Service no-undo.
 
    define private temp-table ttEntity no-undo
        field cEntityName as character
        field oEntity    as Progress.Lang.Object
        .
    
    define private temp-table ttRegisteredAdapters no-undo
        field oAdapter as Progress.Lang.Object
        .
        
    define public property NumRegisteredAdapters as integer no-undo
        get ():
            NumRegisteredAdapters = 0.
            for each ttRegisteredAdapters:
                NumRegisteredAdapters = NumRegisteredAdapters + 1.
            end.
        end get.
        private set.
    
    destructor public Service ():
        /* clean up */
        for each ttEntity:
            delete object ttEntity.oEntity no-error.
            delete ttEntity.
        end.
    end destructor.
    
    constructor private Service():
        /* nothing doing here */
    end constructor.
            
    method public static Service GetInstance():
        if not valid-object(fService) then
            fService = new Service().
            
        return fService.
    end method.        
    
    method public void RegisterServiceAdapter (input poServiceAdapter as ServiceAdapter):
        create ttRegisteredAdapters.
        assign ttRegisteredAdapters.oAdapter = poServiceAdapter.
    end method.

    method public void UnregisterServiceAdapter (input poServiceAdapter as ServiceAdapter):
        find first ttRegisteredAdapters where
                   ttRegisteredAdapters.oAdapter = poServiceAdapter
                   no-error.
        if available ttRegisteredAdapters then
            delete ttRegisteredAdapters.
    end method.
    
    method public BusinessEntity GetEntity(input pcEntity as character):
        def var oBE as BusinessEntity no-undo.
        def var hEntity as handle no-undo.
        
        find first ttEntity where ttEntity.cEntityName = pcEntity no-error.
        if available ttEntity then
            oBE = cast(ttEntity.oEntity, BusinessEntity).
        else
        do:
            case pcEntity:
	            when "CustMaint"  then oBE = new BECustMaint().
                when "PurchOrder" then oBE = new BEPurchaseOrder().
	            when "Department" then oBE = new BEDepartment().
            end case.    /* entity */
            
            if valid-object(oBE) then
            do:
                oBE:FillEntity(True).
                
                create ttEntity.
                ttEntity.cEntityName = oBE:DatasetName.
                ttEntity.oEntity = oBE.
            end.
        end.    /* start entity */
        
        return oBE.
    end method.    /*getEntity*/
    
    method public void saveData(input phDataSet as handle):        
        def var iBuf as int no-undo.
        def var iRow as int no-undo.
        def var hBuffer as handle no-undo.
        def var hBeforeBuffer as handle no-undo.
        def var hQry as handle no-undo.
        def var oBE as BusinessEntity no-undo.
        
        find first ttEntity where ttEntity.cEntityName = phDataset:Name no-error.
        oBE = cast(ttEntity.oEntity, BusinessEntity).
        oBE:AttachDataSources(phDataset).
                
        create query hQry.
        do iBuf = 1 to phDataSet:num-buffers:
            hBuffer = phDataSet:get-buffer-handle(iBuf).
            hBeforeBuffer = hBuffer:before-buffer.
            
            hQry:set-buffers(hBeforeBuffer).
            hQry:query-prepare('for each ' + hBeforeBuffer:name).
            hQry:query-open().
            hQry:get-first().
            do transaction while not hQry:query-off-end:
                if hBeforeBuffer:row-state > 0 then
                    hBeforeBuffer:save-row-changes() no-error.
                hQry:get-next().
            end.
        end.    /* buffer loop */               
        
        oBE:DetachDataSources(phDataset).
        
        delete object hQry no-error.
    end method.    /* savedata */    

END CLASS.